#ifndef __TERMINALS_H
#define __TERMINALS_H

#include "GameStateModule.H"
#include "Game.H"
#include "ServerObjData.H"
#include "memory.H"


#define MAX_TICKS 2400
#define SHOOT_FREQ 4

#define TANK_DMG1 30
#define TANK_DMG2 70
#define MARINE_DMG 6

#define COOLDOWN 8


#define MARINE_RANGE	64
#define TANK_RANGE1 112
#define TANK_RANGE0 32
#define TANK_RANGE2 192

#define MARINE_HP 80
#define TANK_HP 150

#define MARINE_SPEED 3
#define TANK_SPEED 3


/**
 * TERMINALS
 */

#define R_SIZE					23

#define R_TICK					0

#define R_MY_ENERGY				1

#define R_NEAREST_LIGHT_MATE_ENERGY		2
#define R_NEAREST_HEAVY_MATE_ENERGY		3
#define R_NEAREST_LIGHT_ENEMY_ENERGY		4
#define R_NEAREST_HEAVY_ENEMY_ENERGY		5

#define R_WEAKEST_MYRANGE_LIGHT_MATE_ENERGY	6
#define R_WEAKEST_MYRANGE_HEAVY_MATE_ENERGY	7
#define R_WEAKEST_MYRANGE_LIGHT_ENEMY_ENERGY	8
#define R_WEAKEST_MYRANGE_HEAVY_ENEMY_ENERGY	9

#define R_RANGE_ANY_MATE_NUM			10
#define R_RANGE_LIGHT_MATE_NUM			11
#define R_RANGE_HEAVY_MATE_NUM			12
#define R_ALL_LIGHT_MATE_NUM			13
#define R_ALL_HEAVY_MATE_NUM			14

#define R_MYRANGE_ANY_ENEMY_NUM			15
#define R_MYRANGE_LIGHT_ENEMY_NUM		16
#define R_MYRANGE_HEAVY_ENEMY_NUM		17
#define R_RANGE_ANY_ENEMY_NUM			18
#define R_RANGE_LIGHT_ENEMY_NUM			19
#define R_RANGE_HEAVY_ENEMY_NUM			20
#define R_ALL_LIGHT_ENEMY_NUM			21
#define R_ALL_HEAVY_ENEMY_NUM			22



#define V_SIZE			23

#define V_ME		0
#define V_HOME_TEAM	1
#define V_HOME_MINE	2

//furthest marine than nearest tank than nearest siege than in all map
#define V_FURTHEST_RANGE_LIGHT_MATE	3
#define V_FURTHEST_RANGE_HEAVY_MATE	4
#define V_FURTHEST_RANGE_LIGHT_ENEMY	5
#define V_FURTHEST_RANGE_HEAVY_ENEMY	6

//nearest marine than nearest tank than nearest siege than in all map
#define V_NEAREST_LIGHT_MATE	7
#define V_NEAREST_HEAVY_MATE	8
#define V_NEAREST_LIGHT_ENEMY	9
#define V_NEAREST_HEAVY_ENEMY	10

//as above but weakest and in my range
#define V_WEAKEST_MYRANGE_LIGHT_MATE	11
#define V_WEAKEST_MYRANGE_HEAVY_MATE	12
#define V_WEAKEST_MYRANGE_LIGHT_ENEMY	13
#define V_WEAKEST_MYRANGE_HEAVY_ENEMY	14

#define V_CENTER_ALL_MATE		15
#define V_CENTER_LIGHT_MATE		16
#define V_CENTER_HEAVY_MATE		17
#define V_CENTER_ALL_ENEMY		18
#define V_CENTER_LIGHT_ENEMY		19
#define V_CENTER_HEAVY_ENEMY		20

#define V_PATH_MATE			21
#define V_ME_BACK			22


#define B_SIZE			13

#define B_AM_ON_EDGE		0
#define B_AM_ON_FIRE		1
#define B_AM_IN_GROUP		2
#define B_AM_WINNING		3
#define B_AM_BULLY		4
#define B_AM_MOBILE		5
#define B_IS_BEGIN_TIME		6
#define B_AM_HEALTHY		9
#define B_AM_DYING		10
#define B_JUST_SHOT		11
#define B_AM_COLLIDING		12


#define WEAPON_BUFFER		10

struct Tuple {
	double x;
	double y;
};

class Terminals {
	public:
	Terminals() {
		int i = 0;
		for (i = 0; i < R_SIZE; ++i) {
			R[i] = -1;
		}
		for (i = 0; i < V_SIZE; ++i) {
			V[i].x = -1;
			V[i].y = -1;
		}
		home_mine.x = -1;
		home_mine.y = -1;
		home_team.x = -1;
		home_team.y = -1;
		for (i = 0; i < B_SIZE; ++i) {
			B[i] = false;
		}
	}
	
	double R[R_SIZE];
	Tuple V[V_SIZE];
	bool B[B_SIZE];
	
	Tuple home_team;
	Tuple home_mine;

	GameObj* me;
	int range;
	sint4 code;
	sint4 max_speed;

	sint4 hp_team;
	sint4 hp_enemy;
	

	GameObj* nearest_light_mate;
	GameObj* nearest_heavy_mate;
	GameObj* nearest_light_enemy;
	GameObj* nearest_heavy_enemy;
	
	GameObj* weakest_myrange_light_mate;
	GameObj* weakest_myrange_heavy_mate;
	GameObj* weakest_myrange_light_enemy;
	GameObj* weakest_myrange_heavy_enemy;
	
	GameObj* furthest_range_light_mate;
	GameObj* furthest_range_heavy_mate;
	GameObj* furthest_range_light_enemy;
	GameObj* furthest_range_heavy_enemy;
	
	int range_any_mate_num;
	int range_light_mate_num;
	int range_heavy_mate_num;
	int all_light_mate_num;
	int all_heavy_mate_num;

	int myrange_any_enemy_num;
	int myrange_light_enemy_num;
	int myrange_heavy_enemy_num;
	int range_any_enemy_num;
	int range_light_enemy_num;
	int range_heavy_enemy_num;
	int all_light_enemy_num;
	int all_heavy_enemy_num;
	
	Tuple center_all_mate;
	Tuple center_light_mate;
	Tuple center_heavy_mate;

	Tuple center_all_enemy;
	Tuple center_light_enemy;
	Tuple center_heavy_enemy;
	
	char curr_weapon[WEAPON_BUFFER]; 
};

extern class Terminals t;

void show_terminals(Terminals& t);
float get_max_range(GameObj* gob, char* weapon_text);
float get_distance(GameObj* gob1, GameObj* gob2);
void init_terminals(Terminals& t);
void set_global(GameObj* gob, Terminals& t);
void set_me(GameObj* gob, Terminals& t);
void iterate_over_mates(const Game& game, GameObj* gob, Terminals& t);
// call set_me first!!!
void iterate_over_enemies(const Game& game, GameObj* gob, Terminals& t);
void update_terminals(const Game& game, GameObj* gob, Terminals& t);
void update_terminals2(const Game& game, Terminals& t, Memory& m);
Tuple& convert_vector_to_map(GameObj* gob, Terminals& t, Tuple& tp);

#endif
